@@ -1,4 +1,4 @@
-## Copyright (C) 1995, 1996, 1997, 2005, 2007 Kurt Hornik
+## Copyright (C) 1995, 1996, 1997, 2005, 2007, 2008 Kurt Hornik
 ##
 ## This file is part of Octave.
 ##
@@ -25,8 +25,17 @@
 ## Author: KH <Kurt.Hornik@wu-wien.ac.at>
 ## Description: CDF of the binomial distribution
 
+## Deprecated in version 3.0
+
 function cdf = binomial_cdf (varargin)
 
+  persistent warned = false;
+  if (! warned)
+    warned = true;
+    warning ("Octave:deprecated-function",
+             "binomial_cdf is obsolete and will be removed from a future version of Octave; please use binocdf instead");
+  endif
+
  cdf =  binocdf (varargin{:});
 
 endfunction
