@@ -1,5 +1,5 @@
 ## Copyright (C) 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2004, 2005,
-##               2006, 2007 John W. Eaton
+##               2006, 2007, 2008 John W. Eaton
 ## 
 ## This file is part of Octave.
 ##
@@ -166,6 +166,7 @@ function h = imshow (im, varargin)
     tmp = image (round ((rows (colormap ()) - 1) * im));
   endif
   set (gca (), "visible", "off");
+  axis ("image");
 
   if (nargout > 0)
     h = tmp;
