@@ -1,7 +1,7 @@
 /*
 
 Copyright (C) 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002,
-              2003, 2004, 2005, 2006, 2007 John W. Eaton
+              2003, 2004, 2005, 2006, 2007, 2008 John W. Eaton
 
 This file is part of Octave.
 
@@ -3087,7 +3087,7 @@ This function is equivalent to @code{x ./ y}.\n\
 DEFUN (power, args, ,
   "-*- texinfo -*-\n\
 @deftypefn {Built-in Function} {} power (@var{x}, @var{y})\n\
-This function is equivalent to @code{x .\\ y}.\n\
+This function is equivalent to @code{x .^ y}.\n\
 @end deftypefn")
 {
   BINARY_OP_DEFUN_BODY (op_el_pow);
@@ -3096,6 +3096,7 @@ This function is equivalent to @code{x .\\ y}.\n\
 DEFUN (ldivide, args, ,
   "-*- texinfo -*-\n\
 @deftypefn {Built-in Function} {} ldivide (@var{x}, @var{y})\n\
+This function is equivalent to @code{x .\\ y}.\n\
 @end deftypefn")
 {
   BINARY_OP_DEFUN_BODY (op_el_ldiv);
