@@ -1,6 +1,6 @@
 /*
 
-Copyright (C) 2007 John W. Eaton
+Copyright (C) 2007, 2008 John W. Eaton
 
 This file is part of Octave.
 
@@ -26,6 +26,7 @@ along with Octave; see the file COPYING.  If not, see
 
 #include <cctype>
 #include <cfloat>
+#include <cstdlib>
 
 #include <algorithm>
 #include <list>
@@ -454,7 +455,13 @@ gh_manager::get_handle (const std::string& go_name)
 	  handle_free_list.erase (p);
 	}
       else
-	retval = next_handle--;
+	{
+	  static double maxrand = RAND_MAX + 2.0;
+
+	  retval = graphics_handle (next_handle);
+
+	  next_handle = ceil (next_handle) - 1.0 - (rand () + 1.0) / maxrand;
+	}
     }
 
   return retval;
@@ -863,7 +870,8 @@ figure::properties::properties (const graphics_handle& mh,
     currentaxes (octave_NaN),
     colormap (),
     visible ("on"),
-    paperorientation ("portrait")
+    paperorientation ("portrait"),
+    color ( color_values (1, 1, 1))
 { }
 
 void
@@ -922,6 +930,8 @@ figure::properties::set (const caseless_str& name, const octave_value& val)
     set_visible (val);
   else if (name.compare ("paperorientation"))
     set_paperorientation (val);
+  else if (name.compare ("color"))
+    set_color (val);
   else
     {
       modified = false;
@@ -950,6 +960,7 @@ figure::properties::get (void) const
   m.assign ("colormap", colormap);
   m.assign ("visible", visible);
   m.assign ("paperorientation", paperorientation);
+  m.assign ("color", color);
 
   return m;
 }
@@ -985,6 +996,8 @@ figure::properties::get (const caseless_str& name) const
     retval = visible;
   else if (name.compare ("paperorientation"))
     retval = paperorientation;
+  else if (name.compare ("color"))
+    retval = color;
   else
     warning ("get: invalid property `%s'", name.c_str ());
 
@@ -1022,7 +1035,7 @@ figure::properties::factory_defaults (void)
   m["colormap"] = colormap_property ();
   m["visible"] = "on";
   m["paperorientation"] = "portrait";
-
+  m["color"] = color_property (color_values (1, 1, 1));
   return m;
 }
 
@@ -1074,7 +1087,7 @@ default_colororder (void)
 }
 
 axes::properties::properties (const graphics_handle& mh,
-					const graphics_handle& p)
+			      const graphics_handle& p)
   : base_properties (go_name, mh, p),
     position (Matrix ()),
     title (octave_NaN),
@@ -1085,6 +1098,7 @@ axes::properties::properties (const graphics_handle& mh,
     colororder (default_colororder ()),
     dataaspectratio (Matrix (1, 3, 1.0)),
     dataaspectratiomode ("auto"),
+    layer (radio_values ("{bottom}|top")),
     xlim (),
     ylim (),
     zlim (),
@@ -1114,10 +1128,12 @@ axes::properties::properties (const graphics_handle& mh,
     xticklabelmode ("auto"),
     yticklabelmode ("auto"),
     zticklabelmode ("auto"),
-    color (color_values(0, 0, 0), radio_values ("flat|none|interp")),
-    xcolor (),
-    ycolor (),
-    zcolor (),
+    tickdir (radio_values ("{in}|out")),
+    tickdirmode (radio_values ("{auto}|manual")),
+    color (color_values (0, 0, 0), radio_values ("flat|none|interp")),
+    xcolor (color_values (0, 0, 0)),
+    ycolor (color_values (0, 0, 0)),
+    zcolor (color_values (0, 0, 0)),
     xscale (radio_values ("{linear}|log")),
     yscale (radio_values ("{linear}|log")),
     zscale (radio_values ("{linear}|log")),
@@ -1126,6 +1142,7 @@ axes::properties::properties (const graphics_handle& mh,
     zdir ("normal"),
     xaxislocation ("bottom"),
     yaxislocation ("left"),
+    linewidth (0.5),
     view (),
     visible ("on"),
     nextplot ("replace"),
@@ -1250,6 +1267,8 @@ axes::properties::set (const caseless_str& name, const octave_value& val)
     set_dataaspectratio (val);
   else if (name.compare ("dataaspectratiomode"))
     set_dataaspectratiomode (val);
+  else if (name.compare ("layer"))
+    set_layer (val);
   else if (name.compare ("xlim"))
     set_xlim (val);
   else if (name.compare ("ylim"))
@@ -1308,6 +1327,10 @@ axes::properties::set (const caseless_str& name, const octave_value& val)
     set_yticklabelmode (val);
   else if (name.compare ("zticklabelmode"))
     set_zticklabelmode (val);
+  else if (name.compare ("tickdir"))
+    set_tickdir (val);
+  else if (name.compare ("tickdirmode"))
+    set_tickdirmode (val);
   else if (name.compare ("color"))
     set_color (val);
   else if (name.compare ("xcolor"))
@@ -1332,6 +1355,8 @@ axes::properties::set (const caseless_str& name, const octave_value& val)
     set_xaxislocation (val);
   else if (name.compare ("yaxislocation"))
     set_yaxislocation (val);
+  else if (name.compare ("linwdiwth"))
+    set_linewidth (val);
   else if (name.compare ("view"))
     set_view (val);
   else if (name.compare ("visible"))
@@ -1367,6 +1392,7 @@ axes::properties::set_defaults (base_graphics_object& obj,
   colororder = default_colororder ();
   dataaspectratio = Matrix (1, 3, 1.0);
   dataaspectratiomode = "auto";
+  layer = radio_property (radio_values ("{bottom}|top"));
 
   Matrix tlim (1, 2, 0.0);
   tlim(1) = 1;
@@ -1403,6 +1429,8 @@ axes::properties::set_defaults (base_graphics_object& obj,
   xticklabelmode = "auto";
   yticklabelmode = "auto";
   zticklabelmode = "auto";
+  tickdir = radio_property (radio_values ("{in}|out"));
+  tickdirmode = radio_property (radio_values ("{auto}|manual"));
   color = color_property (color_values (0, 0, 0), radio_values("flat|none|interp"));
   xcolor = color_property ("black");
   ycolor = color_property ("black");
@@ -1415,6 +1443,7 @@ axes::properties::set_defaults (base_graphics_object& obj,
   zdir = "normal";
   xaxislocation = "left";
   yaxislocation = "bottom";
+  linewidth = 0.5;
 
   Matrix tview (1, 2, 0.0);
   tview(1) = 90;
@@ -1499,6 +1528,7 @@ axes::properties::get (void) const
   m.assign ("colororder", colororder);
   m.assign ("dataaspectratio", dataaspectratio);
   m.assign ("dataaspectratiomode", dataaspectratiomode);
+  m.assign ("layer", layer);
   m.assign ("xlim", xlim);
   m.assign ("ylim", ylim);
   m.assign ("zlim", zlim);
@@ -1528,6 +1558,8 @@ axes::properties::get (void) const
   m.assign ("xticklabelmode", xticklabelmode);
   m.assign ("yticklabelmode", yticklabelmode);
   m.assign ("zticklabelmode", zticklabelmode);
+  m.assign ("tickdir", tickdir);
+  m.assign ("tickdirmode", tickdirmode);
   m.assign ("color", color);
   m.assign ("xcolor", xcolor);
   m.assign ("ycolor", ycolor);
@@ -1540,6 +1572,7 @@ axes::properties::get (void) const
   m.assign ("zdir", zdir);
   m.assign ("xaxislocation", xaxislocation);
   m.assign ("yaxislocation", yaxislocation);
+  m.assign ("linewidth", linewidth);
   m.assign ("view", view);
   m.assign ("visible", visible);
   m.assign ("nextplot", nextplot);
@@ -1583,6 +1616,8 @@ axes::properties::get (const caseless_str& name) const
     retval = dataaspectratio;
   else if (name.compare ("dataaspectratiomode"))
     retval = dataaspectratiomode;
+  else if (name.compare ("layer"))
+    retval = layer;
   else if (name.compare ("xlim"))
     retval = xlim;
   else if (name.compare ("ylim"))
@@ -1641,6 +1676,10 @@ axes::properties::get (const caseless_str& name) const
     retval = yticklabelmode;
   else if (name.compare ("zticklabelmode"))
     retval = zticklabelmode;
+  else if (name.compare ("tickdir"))
+    retval = tickdir;
+  else if (name.compare ("tickdirmode"))
+    retval = tickdirmode;
   else if (name.compare ("color"))
     retval = color;
   else if (name.compare ("xcolor"))
@@ -1665,6 +1704,8 @@ axes::properties::get (const caseless_str& name) const
     retval = xaxislocation;
   else if (name.compare ("yaxislocation"))
     retval = yaxislocation;
+  else if (name.compare ("linewidth"))
+    retval = linewidth;
   else if (name.compare ("view"))
     retval = view;
   else if (name.compare ("visible"))
@@ -1723,6 +1764,7 @@ axes::properties::factory_defaults (void)
   m["colororder"] = default_colororder ();
   m["dataaspectratio"] = Matrix (1, 3, 1.0);
   m["dataaspectratiomode"] = "auto";
+  m["layer"] = radio_property (radio_values ("{bottom}|top"));
 
   Matrix tlim (1, 2, 0.0);
   tlim(1) = 1;
@@ -1761,6 +1803,8 @@ axes::properties::factory_defaults (void)
   m["xticklabelmode"] = "auto";
   m["yticklabelmode"] = "auto";
   m["zticklabelmode"] = "auto";
+  m["tickdir"] = radio_property (radio_values ("{in}|out"));
+  m["tickdirmode"] = radio_property (radio_values ("{auto}|manual"));
   m["color"] = color_property (color_values (0, 0, 0), radio_values("flat|none|interp"));
   m["xcolor"] = color_property ("black");
   m["ycolor"] = color_property ("black");
@@ -1773,6 +1817,7 @@ axes::properties::factory_defaults (void)
   m["zdir"] = "normal";
   m["xaxislocation"] = "bottom";
   m["yaxislocation"] = "left";
+  m["linewidth"] = 0.5;
 
   Matrix tview (1, 2, 0.0);
   tview(1) = 90;
@@ -1809,29 +1854,19 @@ axes::get_default (const caseless_str& name) const
   return retval;
 }
 
-static void
-check_limit_val (double& min_val, double& max_val, double& min_pos, double val)
-{
-  if (! (xisinf (val) || xisnan (val)))
-    {
-      if (val < min_val)
-	min_val = val;
-
-      if (val > max_val)
-	max_val = val;
-
-      if (val > 0 && val < min_pos)
-	min_pos = val;
-    }
-}
-
 static void
 check_limit_vals (double& min_val, double& max_val, double& min_pos,
 		  const data_property& data)
 {
-  check_limit_val (min_val, max_val, min_pos, data.min_val ());
-  check_limit_val (max_val, max_val, min_pos, data.max_val ());
-  check_limit_val (min_pos, max_val, min_pos, data.min_pos ());
+  double val = data.min_val ();
+  if (! (xisinf (val) || xisnan (val)) && val < min_val)
+    min_val = val;
+  val = data.max_val ();
+  if (! (xisinf (val) || xisnan (val)) && val > max_val)
+    max_val = val;
+  val = data.min_pos ();
+  if (! (xisinf (val) || xisnan (val)) && val > 0 && val < min_pos)
+    min_pos = val;
 }
 
 // Attempt to make "nice" limits from the actual max and min of the
@@ -2117,7 +2152,7 @@ line::properties::properties (const graphics_handle& mh,
     marker ("none"),
     markeredgecolor ("auto"),
     markerfacecolor ("none"),
-    markersize (1),
+    markersize (6),
     keylabel (""),
     interpreter (radio_values ("{tex}|none|latex"))
 { }
@@ -2281,7 +2316,7 @@ line::properties::factory_defaults (void)
   m["marker"] = "none";
   m["markeredgecolor"] = "auto";
   m["markerfacecolor"] = "none";
-  m["markersize"] = 1;
+  m["markersize"] = 6;
   m["keylabel"] = "";
   m["interpreter"] = radio_property (radio_values ("{tex}|none|latex"));
 
@@ -2453,7 +2488,8 @@ image::properties::properties (const graphics_handle& mh,
   : base_properties (go_name, mh, p),
     xdata (Matrix ()),
     ydata (Matrix ()),
-    cdata (Matrix ())
+    cdata (Matrix ()),
+    cdatamapping (radio_values ("{scaled}|direct"))
 { }
 
 void
@@ -2479,6 +2515,8 @@ image::properties::set (const caseless_str& name,
     set_ydata (val);
   else if (name.compare ("cdata"))
     set_cdata (val);
+  else if (name.compare ("cdatamapping"))
+    set_cdatamapping (val);
   else
     {
       modified = false;
@@ -2502,6 +2540,7 @@ image::properties::get (void) const
   m.assign ("xdata", xdata);
   m.assign ("ydata", ydata);
   m.assign ("cdata", cdata);
+  m.assign ("cdatamapping", cdatamapping);
 
   return m;
 }
@@ -2527,6 +2566,8 @@ image::properties::get (const caseless_str& name) const
     retval = ydata;
   else if (name.compare ("cdata"))
     retval = cdata;
+  else if (name.compare ("cdatamapping"))
+    retval = cdatamapping;
   else
     warning ("get: invalid property `%s'", name.c_str ());
 
@@ -2541,6 +2582,7 @@ image::properties::factory_defaults (void)
   m["xdata"] = Matrix ();
   m["ydata"] = Matrix ();
   m["cdata"] = Matrix ();
+  m["cdatamapping"] = radio_property (radio_values ("{scaled}|direct"));
 
   return m;
 }
@@ -2556,17 +2598,18 @@ patch::properties::properties (const graphics_handle& mh,
     ydata (Matrix ()),
     zdata (Matrix ()),
     cdata (Matrix ()),
+    cdatamapping (radio_values ("{scaled}|direct")),
     faces (Matrix ()),
     vertices (Matrix ()),
     facecolor (radio_values ("{flat}|none|interp")),
     facealpha (1.0),
-    edgecolor (color_values(0, 0, 0), radio_values ("flat|none|interp")),
+    edgecolor (color_values (0, 0, 0), radio_values ("flat|none|interp")),
     linestyle ("-"),
     linewidth (0.5),
     marker ("none"),
     markeredgecolor ("auto"),
     markerfacecolor ("none"),
-    markersize (1),
+    markersize (6),
     keylabel (""),
     interpreter (radio_values ("{tex}|none|latex"))
 { }
@@ -2596,6 +2639,8 @@ patch::properties::set (const caseless_str& name,
     set_zdata (val);
   else if (name.compare ("cdata"))
     set_cdata (val);
+  else if (name.compare ("cdatamapping"))
+    set_cdatamapping (val);
   else if (name.compare ("faces"))
     set_faces (val);
   else if (name.compare ("vertices"))
@@ -2646,6 +2691,7 @@ patch::properties::get (void) const
   m.assign ("ydata", ydata);
   m.assign ("zdata", zdata);
   m.assign ("cdata", cdata);
+  m.assign ("cdatamapping", cdatamapping);
   m.assign ("faces", faces);
   m.assign ("vertices", vertices);
   m.assign ("facecolor", facecolor);
@@ -2686,6 +2732,8 @@ patch::properties::get (const caseless_str& name) const
     retval = zdata;
   else if (name.compare ("cdata"))
     retval = cdata;
+  else if (name.compare ("cdatamapping"))
+    retval = cdatamapping;
   else if (name.compare ("faces"))
     retval = faces;
   else if (name.compare ("vertices"))
@@ -2694,7 +2742,7 @@ patch::properties::get (const caseless_str& name) const
     retval = facecolor;
   else if (name.compare ("facealpha"))
     retval = facealpha;
-  else if (name.compare ("egdecolor"))
+  else if (name.compare ("edgecolor"))
     retval = edgecolor;
   else if (name.compare ("linestyle"))
     retval = linestyle;
@@ -2727,6 +2775,7 @@ patch::properties::factory_defaults (void)
   m["ydata"] = Matrix ();
   m["zdata"] = Matrix ();
   m["cdata"] = Matrix ();
+  m["cdatamapping"] = radio_property (radio_values ("{scaled}|direct"));
   m["faces"] = Matrix ();
   m["vertices"] = Matrix ();
   m["facecolor"] = color_property ();
@@ -2737,7 +2786,7 @@ patch::properties::factory_defaults (void)
   m["marker"] = "none";
   m["markeredgecolor"] = "auto";
   m["markerfacecolor"] = "none";
-  m["markersize"] = 1;
+  m["markersize"] = 6;
   m["keylabel"] = "";
   m["interpreter"] = radio_property (radio_values ("{tex}|none|latex"));
 
@@ -2755,15 +2804,16 @@ surface::properties::properties (const graphics_handle& mh,
     ydata (Matrix ()),
     zdata (Matrix ()),
     cdata (Matrix ()),
+    cdatamapping (radio_values ("{scaled}|direct")),
     facecolor (radio_values ("{flat}|none|interp")),
     facealpha (1.0),
-    edgecolor (color_values(0, 0, 0), radio_values ("flat|none|interp")),
+    edgecolor (color_values (0, 0, 0), radio_values ("flat|none|interp")),
     linestyle ("-"),
     linewidth (0.5),
     marker ("none"),
     markeredgecolor ("auto"),
     markerfacecolor ("none"),
-    markersize (1),
+    markersize (6),
     keylabel (""),
     interpreter (radio_values ("{tex}|none|latex"))
 { }
@@ -2793,6 +2843,8 @@ surface::properties::set (const caseless_str& name,
     set_zdata (val);
   else if (name.compare ("cdata"))
     set_cdata (val);
+  else if (name.compare ("cdatamapping"))
+    set_cdatamapping (val);
   else if (name.compare ("facecolor"))
     set_facecolor (val);
   else if (name.compare ("facealpha"))
@@ -2839,6 +2891,7 @@ surface::properties::get (void) const
   m.assign ("ydata", ydata);
   m.assign ("zdata", zdata);
   m.assign ("cdata", cdata);
+  m.assign ("cdatamapping", cdatamapping);
   m.assign ("facecolor", facecolor);
   m.assign ("facealpha", facealpha);
   m.assign ("edgecolor", edgecolor);
@@ -2877,6 +2930,8 @@ surface::properties::get (const caseless_str& name) const
     retval = zdata;
   else if (name.compare ("cdata"))
     retval = cdata;
+  else if (name.compare ("cdatamapping"))
+    retval = cdatamapping;
   else if (name.compare ("facecolor"))
     retval = facecolor;
   else if (name.compare ("facealpha"))
@@ -2914,6 +2969,7 @@ surface::properties::factory_defaults (void)
   m["ydata"] = Matrix ();
   m["zdata"] = Matrix ();
   m["cdata"] = Matrix ();
+  m["cdatamapping"] = radio_property (radio_values ("{scaled}|direct"));
   m["facecolor"] = color_property ();
   m["facealpha"] = 1.0;
   m["edgecolor"] = color_property ("black");
@@ -2922,7 +2978,7 @@ surface::properties::factory_defaults (void)
   m["marker"] = "none";
   m["markeredgecolor"] = "auto";
   m["markerfacecolor"] = "none";
-  m["markersize"] = 1;
+  m["markersize"] = 6;
   m["keylabel"] = "";
   m["interpreter"] = radio_property (radio_values ("{tex}|none|latex"));
 
@@ -2950,8 +3006,11 @@ base_graphics_object::get_factory_default (const caseless_str& name) const
   return parent_obj.get_factory_default (type () + name);
 }
 
+// We use a random value for the handle to avoid issues with plots and
+// scalar values for the first argument.
 gh_manager::gh_manager (void)
-  : handle_map (), handle_free_list (), next_handle (-1)
+  : handle_map (), handle_free_list (),
+    next_handle (-1.0 - (rand () + 1.0) / (RAND_MAX + 2.0))
 {
   handle_map[0] = graphics_object (new root_figure ());
 }
@@ -3197,6 +3256,9 @@ make_graphics_object (const std::string& go_name,
 	      xset (h, args.splice (0, 1));
 
 	      retval = h.value ();
+
+	      if (! error_state)
+		feval ("__request_drawnow__");
 	    }
 	  else
 	    error ("__go%s__: unable to create graphics handle",
