@@ -1,13 +1,13 @@
 /*
 
-Copyright (C) 1996, 1997 John W. Eaton
+Copyright (C) 1996, 1997, 2003, 2004, 2005, 2006, 2007 John W. Eaton
 
 This file is part of Octave.
 
 Octave is free software; you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
-Free Software Foundation; either version 2, or (at your option) any
-later version.
+Free Software Foundation; either version 3 of the License, or (at your
+option) any later version.
 
 Octave is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
@@ -15,9 +15,8 @@ FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
-along with Octave; see the file COPYING.  If not, write to the Free
-Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
-02110-1301, USA.
+along with Octave; see the file COPYING.  If not, see
+<http://www.gnu.org/licenses/>.
 
 */
 
@@ -40,7 +39,6 @@ Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 #include "file-ops.h"
 #include "glob-match.h"
 #include "lo-mappers.h"
-#include "lo-sstream.h"
 #include "mach-info.h"
 #include "oct-env.h"
 #include "oct-time.h"
@@ -110,9 +108,9 @@ read_mat_binary_data (std::istream& is, double *data, int precision,
 }
 
 int
-read_mat_file_header (std::istream& is, bool& swap, FOUR_BYTE_INT& mopt, 
-		      FOUR_BYTE_INT& nr, FOUR_BYTE_INT& nc,
-		      FOUR_BYTE_INT& imag, FOUR_BYTE_INT& len,
+read_mat_file_header (std::istream& is, bool& swap, int32_t& mopt, 
+		      int32_t& nr, int32_t& nc,
+		      int32_t& imag, int32_t& len,
 		      int quiet)
 {
   swap = false;
@@ -121,20 +119,20 @@ read_mat_file_header (std::istream& is, bool& swap, FOUR_BYTE_INT& mopt,
   // being able to read another mopt value should not result in an
   // error.
 
-  is.read (X_CAST (char *, &mopt), 4);
+  is.read (reinterpret_cast<char *> (&mopt), 4);
   if (! is)
     return 1;
 
-  if (! is.read (X_CAST (char *, &nr), 4))
+  if (! is.read (reinterpret_cast<char *> (&nr), 4))
     goto data_read_error;
 
-  if (! is.read (X_CAST (char *, &nc), 4))
+  if (! is.read (reinterpret_cast<char *> (&nc), 4))
     goto data_read_error;
 
-  if (! is.read (X_CAST (char *, &imag), 4))
+  if (! is.read (reinterpret_cast<char *> (&imag), 4))
     goto data_read_error;
 
-  if (! is.read (X_CAST (char *, &len), 4))
+  if (! is.read (reinterpret_cast<char *> (&len), 4))
     goto data_read_error;
 
 // If mopt is nonzero and the byte order is swapped, mopt will be
@@ -277,7 +275,7 @@ read_mat_binary_data (std::istream& is, const std::string& filename,
   int mach = 0;
   int dlen = 0;
 
-  FOUR_BYTE_INT mopt, nr, nc, imag, len;
+  int32_t mopt, nr, nc, imag, len;
 
   int err = read_mat_file_header (is, swap, mopt, nr, nc, imag, len);
   if (err)
@@ -323,7 +321,7 @@ read_mat_binary_data (std::istream& is, const std::string& filename,
   {
     OCTAVE_LOCAL_BUFFER (char, name, len+1);
     name[len] = '\0';
-    if (! is.read (X_CAST (char *, name), len))
+    if (! is.read (name, len))
       goto data_read_error;
     retval = name;
 
@@ -390,7 +388,7 @@ bool
 save_mat_binary_data (std::ostream& os, const octave_value& tc,
 		      const std::string& name) 
 {
-  FOUR_BYTE_INT mopt = 0;
+  int32_t mopt = 0;
 
   mopt += tc.is_string () ? 1 : 0;
 
@@ -399,25 +397,25 @@ save_mat_binary_data (std::ostream& os, const octave_value& tc,
 
   mopt += 1000 * float_format_to_mopt_digit (flt_fmt);
 
-  os.write (X_CAST (char *, &mopt), 4);
+  os.write (reinterpret_cast<char *> (&mopt), 4);
   
-  FOUR_BYTE_INT nr = tc.rows ();
-  os.write (X_CAST (char *, &nr), 4);
+  int32_t nr = tc.rows ();
+  os.write (reinterpret_cast<char *> (&nr), 4);
 
-  FOUR_BYTE_INT nc = tc.columns ();
-  os.write (X_CAST (char *, &nc), 4);
+  int32_t nc = tc.columns ();
+  os.write (reinterpret_cast<char *> (&nc), 4);
 
   octave_idx_type len = nr * nc;
 
-  FOUR_BYTE_INT imag = tc.is_complex_type () ? 1 : 0;
-  os.write (X_CAST (char *, &imag), 4);
+  int32_t imag = tc.is_complex_type () ? 1 : 0;
+  os.write (reinterpret_cast<char *> (&imag), 4);
 
   // LEN includes the terminating character, and the file is also
   // supposed to include it.
 
-  FOUR_BYTE_INT name_len = name.length () + 1;
+  int32_t name_len = name.length () + 1;
 
-  os.write (X_CAST (char *, &name_len), 4);
+  os.write (reinterpret_cast<char *> (&name_len), 4);
   os << name << '\0';
 
   if (tc.is_string ())
@@ -439,7 +437,7 @@ save_mat_binary_data (std::ostream& os, const octave_value& tc,
 	  for (octave_idx_type j = 0; j < ncol; j++)
 	    buf[j*nrow+i] = static_cast<double> (*s++ & 0x00FF);
        	}
-      os.write ((char *)buf, nrow*ncol*sizeof(double));
+      os.write (reinterpret_cast<char *> (buf), nrow*ncol*sizeof(double));
       
       unwind_protect::run_frame ("save_mat_binary_data");
     }
@@ -452,31 +450,31 @@ save_mat_binary_data (std::ostream& os, const octave_value& tc,
       for (octave_idx_type i = 0; i < nel; i++)
 	{
 	  double x = base + i * inc;
-	  os.write (X_CAST (char *, &x), 8);
+	  os.write (reinterpret_cast<char *> (&x), 8);
 	}
     }
   else if (tc.is_real_scalar ())
     {
       double tmp = tc.double_value ();
-      os.write (X_CAST (char *, &tmp), 8);
+      os.write (reinterpret_cast<char *> (&tmp), 8);
     }
   else if (tc.is_real_matrix ())
     {
       Matrix m = tc.matrix_value ();
-      os.write (X_CAST (char *, m.data ()), 8 * len);
+      os.write (reinterpret_cast<const char *> (m.data ()), 8 * len);
     }
   else if (tc.is_complex_scalar ())
     {
       Complex tmp = tc.complex_value ();
-      os.write (X_CAST (char *, &tmp), 16);
+      os.write (reinterpret_cast<char *> (&tmp), 16);
     }
   else if (tc.is_complex_matrix ())
     {
       ComplexMatrix m_cmplx = tc.complex_matrix_value ();
       Matrix m = ::real (m_cmplx);
-      os.write (X_CAST (char *, m.data ()), 8 * len);
+      os.write (reinterpret_cast<const char *> (m.data ()), 8 * len);
       m = ::imag (m_cmplx);
-      os.write (X_CAST (char *, m.data ()), 8 * len);
+      os.write (reinterpret_cast<const char *> (m.data ()), 8 * len);
     }
   else
     gripe_wrong_type_arg ("save", tc, false);
