@@ -1,7 +1,7 @@
 /*
 
 Copyright (C) 1996, 1997, 1998, 2000, 2002, 2003, 2004, 2005, 2006,
-              2007 John W. Eaton
+              2007, 2008 John W. Eaton
 
 This file is part of Octave.
 
@@ -118,7 +118,7 @@ octave_struct::subsref (const std::string& type,
 
 	    if (! error_state)
 	      {
-		Cell t = tmp.index (idx.front ());
+		Cell t = tmp.index (idx.front (), true);
 
 		retval(0) = (t.length () == 1) ? t(0) : octave_value (t, true);
 
@@ -129,7 +129,7 @@ octave_struct::subsref (const std::string& type,
 	      }
 	  }
 	else
-	  retval(0) = map.index (idx.front ());
+	  retval(0) = map.index (idx.front (), true);
       }
       break;
 
