@@ -207,13 +207,21 @@ public:
   octave_value (const octave_uint32& i);
   octave_value (const octave_uint64& i);
   octave_value (const int8NDArray& inda);
+  octave_value (const ArrayN<octave_int8>& inda);
   octave_value (const int16NDArray& inda);
+  octave_value (const ArrayN<octave_int16>& inda);
   octave_value (const int32NDArray& inda);
+  octave_value (const ArrayN<octave_int32>& inda);
   octave_value (const int64NDArray& inda);
+  octave_value (const ArrayN<octave_int64>& inda);
   octave_value (const uint8NDArray& inda);
+  octave_value (const ArrayN<octave_uint8>& inda);
   octave_value (const uint16NDArray& inda);
+  octave_value (const ArrayN<octave_uint16>& inda);
   octave_value (const uint32NDArray& inda);
+  octave_value (const ArrayN<octave_uint32>& inda);
   octave_value (const uint64NDArray& inda);
+  octave_value (const ArrayN<octave_uint64>& inda);
   octave_value (double base, double limit, double inc);
   octave_value (const Range& r);
   octave_value (const Octave_map& m);
